# -*- coding: utf-8 -*-
"""
Created on Mon Mar 25

@author: hannah janout
"""
from PyQt5 import QtCore, QtGui, QtWidgets
import ROI
import Communicate
import CrossSection as cs
import ScaleMode

# class which expands a normal QGraphicsScene


class GraphicsSceneV(QtWidgets.QGraphicsScene):

    coord = list()
    add = False
    rect = True
    points = []
    pen = QtGui.QPen(QtCore.Qt.red)
    brush = QtGui.QBrush()
    roi = []
    setbackground = False
    backgroundCoord = list()
    scale = 1
    scaleLength = 1
    scaleLines = list()
    pixGray = list()
    pixColor = list()
    pixThresh = list()
    crossSec = False
    cross = []
    crossSections = list()
    originalWidth = 0
    originalHeight = 0
    scaleMode = ScaleMode.ScaleMode.BottomRight
    effect = QtWidgets.QGraphicsColorizeEffect()
    minArea = 0

    def __init__(self, parent=None):
        QtWidgets.QGraphicsScene.__init__(self, parent)
        self.c = Communicate.Communicate()
        self.pixColor = list(self.pixColor)
        self.pixGray = list(self.pixGray)
        self.pixThresh = list(self.pixThresh)

# overridden functions+++++++++++++++++++++++++++++++++++++++++++++++++++#
    def mousePressEvent(self, event): 
        if self.setbackground and event.scenePos().x() <= self.sceneRect().width() and event.scenePos().y() <= self.sceneRect().height(): 
            self.backgroundCoord.append(event.scenePos().x())
            self.backgroundCoord.append(event.scenePos().y()) 
            self.points.append(self.addRect(event.scenePos().x(), event.scenePos().y(), 1, 1, self.pen, self.brush))
            if len(self.backgroundCoord) == 4:
                self.c.setBackground.emit()
        if self.add and event.scenePos().x() <= self.sceneRect().width() and event.scenePos().y() <= self.sceneRect().height():
            self.coord.append(event.scenePos().x())
            self.coord.append(event.scenePos().y()) 
            self.points.append(self.addRect(event.scenePos().x(), event.scenePos().y(), 1, 1, self.pen, self.brush))
            if self.rect and len(self.coord) == 4:
                if self.coord[0] < self.coord[2] and self.coord[1] < self.coord[3]:
                    self.c.setWorking.emit()
                    r = ROI.RectROI(self.addRect(QtCore.QRectF(self.coord[0], self.coord[1], abs(self.coord[2] - self.coord[0]), abs(self.coord[3] - self.coord[1])), self.pen, self.brush))                
                    r.originalCoord.append(QtCore.QPoint(self.coord[0] * (1/self.scale), self.coord[1] * (1/self.scale)))
                    r.originalCoord.append(QtCore.QPoint(self.coord[2] * (1/self.scale), self.coord[3] * (1/self.scale)))
                    r.adjustAfterSetOriginal()
                    self.roi.append(r)
                elif self.coord[0] > self.coord[2] and self.coord[1] > self.coord[3]:
                    r = ROI.RectROI(self.addRect(QtCore.QRectF(self.coord[2], self.coord[3], abs(self.coord[2] - self.coord[0]), abs(self.coord[3] - self.coord[1])), self.pen, self.brush))
                    r.originalCoord.append(QtCore.QPoint(self.coord[2] * (1 / self.scale), self.coord[3] * (1 / self.scale)))
                    r.originalCoord.append(QtCore.QPoint(self.coord[0] * (1 / self.scale), self.coord[1] * (1 / self.scale)))
                    self.roi.append(r)
                    r.adjustAfterSetOriginal()
                self.deletePoints()
                self.clearCoord()           
                self.add = False
                self.c.createROI.emit()
                self.c.setIdle.emit()
            elif not self.rect and len(self.coord) > 4 and self.finishedPoly():                
                poly = QtGui.QPolygonF()
                self.c.setWorking.emit()
                for i in range(0, len(self.coord)-1, 2):
                    point = QtCore.QPoint(self.coord[i], self.coord[i+1])
                    poly.append(point)
                p = ROI.PolyROI(self.addPolygon(poly, self.pen, self.brush), self.scale)
                self.roi.append(p)
                self.deletePoints()
                self.clearCoord()           
                self.add = False
                self.c.createROI.emit()
                self.c.setIdle.emit()

        if self.crossSec:
            self.cross.append(event.scenePos().x())
            self.cross.append(event.scenePos().y())
            self.points.append(self.addRect(event.scenePos().x(), event.scenePos().y(), 1, 1, self.pen, self.brush))
            if len(self.cross) == 4:
                self.crossSections.append(cs.CrossSection(self.addLine(self.cross[0], self.cross[1], self.cross[2], self.cross[3], self.pen), self.scale))
                self.deletePoints()
                self.c.crossSection.emit()

    def mouseMoveEvent(self, event):
        if event.scenePos().x() <= self.originalWidth*self.scale and event.scenePos().y() <= self.originalHeight*self.scale:
            self.c.mouseMove.emit(int(event.scenePos().x()), int(event.scenePos().y()))

# getter+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def getLatestLine(self):
        return self.crossSections[len(self.crossSections)-1]

    def getCrossSecCoord(self):
        return self.cross

    def getScale(self):
        return self.scale

    def getScaleLen(self):
        return self.scaleLength

    def getPixColor(self):
        return self.pixColor

    def getPixGray(self):
        return self.pixGray

    def getPixColorAt(self, i):
        return self.pixColor[i]

    def getPixGrayAt(self, i):
        return self.pixGray[i]

    def getCrossSections(self):
        return self.crossSections

    def getOrigHeight(self):
        return self.originalHeight

    def getOrigWidth(self):
        return self.originalWidth

    def getPixThresh(self):
        return self.pixThresh

    def getPixThreshAt(self, i):
        return self.pixThresh[i]

    def getMinArea(self):
        return self.minArea
# setters++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def clearPixmaps(self):
        self.pixGray = list()
        self.pixColor = list()
        self.pixThresh = list()

    def setScaleMode(self, mode):
        if mode == 0:
            self.scaleMode = ScaleMode.ScaleMode.BottomRight
        elif mode == 1:
            self.scaleMode = ScaleMode.ScaleMode.BottomLeft
        elif mode == 2:
            self.scaleMode = ScaleMode.ScaleMode.TopRight
        elif mode == 3:
            self.scaleMode = ScaleMode.ScaleMode.TopLeft

    def setCoordinates(self, p):
        for i in range(0, len(self.coord)-1, 2):
            p.appendX(self.coord[i] * (1 / self.scale))
            p.appendY(self.coord[i+1] * (1 / self.scale))
            p.originalCoordappend(QtCore.QPoint(self.coord[i] * (1 / self.scale), self.coord[i+1] * (1 / self.scale)))
        p.initializeOriginalXY()
        p.initializeHeight()
        p.initializeWidth()

    def setColoredQimg(self, qimg):
        self.colorQimg = qimg

    def setScale(self, scale):
        self.scale = scale

    def setScaleLen(self, scaleLen):
        self.scaleLength = scaleLen

    def setPixColor(self, pc):
        self.pixColor = pc

    def setPixGray(self, pg):
        self.pixGray = pg

    def setPixThresh(self, pt):
        self.pixThresh = pt

    def setOrigHeight(self, h):
        self.originalHeight = h

    def setOrigWidth(self, w):
        self.originalWidth = w

    def setMinArea(self, min):
        self.minArea = min

    def addRectangle(self, coords1, coords2):
        r = ROI.RectROI(self.addRect(QtCore.QRectF(coords1, coords2), self.pen, self.brush))
        self.roi.append(r)
        self.c.createROI.emit()

    def addPoly(self, coords, w, h):
        poly = QtGui.QPolygonF()
        for i in range(1, len(coords)):
            c = coords[i]
            point = QtCore.QPoint(int(c[0]) * self.scale, int(c[1]) * self.scale)
            poly.append(point)
            self.coord.append(int(c[0] * self.scale))
            self.coord.append(int(c[1] * self.scale))
        if poly.boundingRect().width() >= w and poly.boundingRect().height() >= h:
            p = ROI.PolyROI(self.addPolygon(poly, self.pen, self.brush), self.scale)
            self.roi.append(p)
            self.clearCoord()
        self.c.createROI.emit()

    def createScale(self, drawScale):
        text = self.addText(str(self.scaleLength) + " µm")
        if drawScale and (self.scale * self.scaleLength) <= self.sceneRect().width():
            if self.scaleMode == ScaleMode.ScaleMode.BottomRight:
                x1 = self.sceneRect().width() - 30 - (self.scale * self.scaleLength)
                y = self.sceneRect().height() - 30
                x2 = self.sceneRect().width() - 30
                text.setPos(self.sceneRect().width() - 100, y + 2)
            elif self.scaleMode == ScaleMode.ScaleMode.BottomLeft:
                x1 = 30
                y = self.sceneRect().height() - 30
                x2 = (self.scale * self.scaleLength) + 30
                text.setPos(30, y + 2)
            elif self.scaleMode == ScaleMode.ScaleMode.TopRight:
                x1 = self.sceneRect().width() - 30 - (self.scale * self.scaleLength)
                y = 30
                x2 = self.sceneRect().width() - 30
                text.setPos(self.sceneRect().width() - 100, y + 2)
            else:
                x1 = 30
                y = 30
                x2 = (self.scale * self.scaleLength) + 30
                text.setPos(30, y + 2)
            text.setDefaultTextColor(QtGui.QColor(255, 255, 255))
            text.setTextWidth(100.0)
            self.scaleLines.append(self.addLine(x1, y, x2, y, self.pen))
            self.scaleLines.append(self.addLine(x1, y + 2, x1, y - 2, self.pen))
            self.scaleLines.append(self.addLine(x2, y + 2, x2, y - 2, self.pen))
            self.scaleLines.append(text)

    def addPixColor(self, p):
        self.pixColor.append(p)

    def addPixGray(self, p):
        self.pixGray.append(p)

    def addPixColorAt(self, p, nr):
        if nr == 0:
            self.pixColor.append(p)

    def appendPixThresh(self, t):
        self.pixThresh.append(t)

    def addPixGrayAt(self, p, nr):
        if nr == 0:
            self.pixGray.append(p)
        elif nr == 1:
            self.pixGray2.append(p)
        elif nr == 2:
            self.pixGray3.append(p)
        elif nr == 1:
            self.pixGray4.append(p)

    def deletePoints(self):
        for x in range(0, len(self.points)):
            self.removeItem(self.points[x])
        self.points.clear()

    def clearCoord(self):
        self.coord.clear()

    def removeAllCrossSec(self):
        for c in self.crossSections:
            self.removeItem(c.getLine())

    def deleteAllROI(self):
        for r in self.roi:
            self.removeROI(r)

    def removeROI(self, r):
        if r.getType().name is "Rectangle":
            self.removeItem(r.getRect())
        else:
            self.removeItem(r.getPoly())

    def removeScale(self):
        for x in range(0, len(self.scaleLines)):
            self.removeItem(self.scaleLines[x])
        self.scaleLines.clear()

    # redraw
    def redrawROI(self, ratio):
        newROI = []
        self.scale = ratio
        for i in range(0, len(self.roi)):
            if self.roi[i].getType().name is "Rectangle":
                tl = self.roi[i].getOriginalCoord()[0]
                br = self.roi[i].getOriginalCoord()[1]
                newr = ROI.RectROI(self.addRect(
                    QtCore.QRectF(QtCore.QPoint(int(tl.x()*self.scale), int(tl.y()*self.scale)), QtCore.QPoint(int(br.x()*self.scale), int(br.y()*self.scale))), self.pen, self.brush))
                newr.setOriginalCoord(list(self.roi[i].getOriginalCoord()))
                newr.adjustAfterSetOriginal()
                newROI.append(newr)
            else:
                poly = QtGui.QPolygonF()
                ori = self.roi[i].getOriginalCoord()
                for i in range(0, self.roi[i].getOriginalPoly().polygon().count()):
                    p = ori[i]
                    point = QtCore.QPoint(int(p.x()*self.scale), int(p.y()*self.scale))
                    poly.append(point)
                newr = ROI.PolyROI(self.addPolygon(poly, self.pen, self.brush), self.scale)

                newr.setAfterCopy(self.scale)
                newROI.append(newr)
        self.deleteAllROI()
        self.roi = newROI

    def redrawScalebar(self):
        self.removeScale()
        self.createScale(True)

    def redrawCrossSec(self):
        newcross = []
        for i in range(0, len(self.crossSections)):
            coord = []
            coord = self.crossSections[i].getOriginalCoord()
            newcross.append(cs.CrossSection(self.addLine(coord[0].x() * self.scale, coord[0].y() * self.scale, coord[1].x() * self.scale, coord[1].y() * self.scale, self.pen), self.scale))
        self.removeAllCrossSec()
        self.crossSections = newcross
        self.c.crossSectionRedraw.emit()

    def finishedPoly(self):
        return abs(self.coord[0] - self.coord[len(self.coord)-2]) < 10 and abs(self.coord[1] - self.coord[len(self.coord)-1]) < 10

    def convertColoredQimg(self):
        self.grayQimg = self.colorQimg
        for x in range(0, self.colorQimg.width()):
            for y in range(0, self.colorQimg.heigth()):
                p = QtGui.QColor.QGray(self.grayQimg.pixel(x,y))
                self.grayQimg.setPixel(x, y, QtGui.QColor(p, p, p).rgb())

# unused+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def redraw(self, color):
        if color: 
            self.pen = QtGui.QPen(QtCore.Qt.white)
        else:
            self.pen = QtGui.QPen(QtCore.Qt.yellow)
            
        for x in range(0, len(self.roi)):
            if self.roi[x].getType().name is ROI.ROIType.Rectangle.name:
                self.addRect(self.roi[x].getRect().rect(), self.pen, self.brush)
            else:
                self.addPolygon(self.roi[x].getPoly().polygon(), self.pen, self.brush)


class GraphicsSceneScale(QtWidgets.QGraphicsScene):

    coord = []
    add = False
    rect = True
    points = []
    pen = QtGui.QPen(QtCore.Qt.black)
    brush = QtGui.QBrush()
    scaleLength = 1
    scaleLines = []
    scale = 1
    scaleMode = ScaleMode.ScaleMode.BottomRight
    imageCoords = []
    ratio = 1

    def __init__(self, parent=None):
        QtWidgets.QGraphicsScene.__init__(self, parent)
        self.c = Communicate.Communicate()
        
# getters++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def getScale(self):
        return self.scale

    def getRatio(self):
        return self.ratio

    def getImageCoords(self):
        return self.imageCoords

# setters++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#
    def setScaleMode(self, mode):
        if mode == 0:
            self.scaleMode = ScaleMode.ScaleMode.BottomRight
        elif mode == 1:
            self.scaleMode = ScaleMode.ScaleMode.BottomLeft
        elif mode == 2:
            self.scaleMode = ScaleMode.ScaleMode.TopRight
        elif mode == 3:
            self.scaleMode = ScaleMode.ScaleMode.TopLeft

    def setRatio(self, ratio):
        self.ratio = ratio

    def setScale(self, scale):
        self.scale = scale

    def setScaleLen(self, scaleLen):
        self.scaleLength = scaleLen

    def setImageCoords(self, coords):
        self.imageCoords = coords



# helpers++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++#

    def clearCoord(self):
        self.coord.clear()

    def createScale(self, drawScale):
        if drawScale and (self.scale * self.scaleLength * self.ratio) <= self.sceneRect().width():
            text = self.addText(str(self.scaleLength) + " µm")
            if self.scaleMode == ScaleMode.ScaleMode.BottomRight:
                if self.imageCoords[1].x() <= 791:
                    x1 = self.imageCoords[1].x() - 10 - (self.scale * self.scaleLength * self.ratio)
                    x2 = self.imageCoords[1].x() - 10
                else:
                    x1 = 750 - (self.scale * self.scaleLength * self.ratio)
                    x2 = 750

            elif self.scaleMode == ScaleMode.ScaleMode.BottomLeft:
                x1 = 30
                x2 = self.scale * self.scaleLength * self.ratio + 30
            elif self.scaleMode == ScaleMode.ScaleMode.TopRight:
                if self.imageCoords[1].x() <= 791:
                    x1 = self.imageCoords[1].x() - 10 - (self.scale * self.scaleLength * self.ratio)
                    x2 = self.imageCoords[1].x() - 10
                else:
                    x1 = 750 - (self.scale * self.scaleLength * self.ratio)
                    x2 = 750
            else:
                x1 = 30
                x2 = self.scale * self.scaleLength * self.ratio + 30
            text.setDefaultTextColor(QtGui.QColor(0, 0, 0))
            text.adjustSize()
            if self.scaleMode == ScaleMode.ScaleMode.TopRight or self.scaleMode == ScaleMode.ScaleMode.TopLeft:
                if self.scaleMode == ScaleMode.ScaleMode.TopLeft:
                    text.setPos(x1, 5)
                else:
                    text.setPos(x2-int(text.textWidth()), 5)
                y = 30
            else:
                y = 10
                if self.scaleMode == ScaleMode.ScaleMode.BottomLeft:
                    text.setPos(x1, y + 2)
                else:
                    text.setPos(x2-int(text.textWidth()), y + 2)
            self.scaleLines.append(text)
            self.scaleLines.append(self.addLine(x1, y, x2, y, self.pen))
            self.scaleLines.append(self.addLine(x1, y + 2, x1, y - 2, self.pen))
            self.scaleLines.append(self.addLine(x2, y + 2, x2, y - 2, self.pen))
        elif not (self.scale * self.scaleLength * self.ratio) <= self.sceneRect().width():
            self.c.scaleWarning.emit()
        self.c.scalebarChangePosition.emit()

    def removeScale(self):
        for x in range(0, len(self.scaleLines)):
            self.removeItem(self.scaleLines[x])
        self.scaleLines.clear()

    def redrawScalebar(self):
        self.removeScale()
        self.createScale(True)


    def positionScalebar(self):
        self.c.scalebarChangePosition.emit()